# -*- coding: utf-8 -*-

"""Kleides Multi Factor Authentication."""

__author__ = """Harm Geerts"""
__email__ = 'hgeerts@osso.nl'
__version__ = '0.1.4'

default_app_config = 'kleides_mfa.apps.KleidesMfaConfig'
